package de.mdelab.mlsdm.examples.elevator;


import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.XMLResource;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

import de.mdelab.mlcallactions.MlcallactionsPackage;
import de.mdelab.mlcore.MlcorePackage;
import de.mdelab.mlexpressions.MlexpressionsPackage;
import de.mdelab.mlsdm.Activity;
import de.mdelab.mlsdm.MlsdmPackage;
import de.mdelab.mlsdm.interpreter.debug.MLSDMInterpreterLauncher;
import de.mdelab.mlstorypatterns.MlstorypatternsPackage;
import de.mdelab.sdm.interpreter.core.SDMException;
import de.mdelab.sdm.interpreter.core.variables.Variable;

public class LaunchSdmElevatorJavaApp {
	
	private static final URI BASE_URI = URI.createFileURI( new File(".").getAbsolutePath() );
	
	private final URI sdResUri;
	private final URI buildingModelResUri;
	
	private LaunchSdmElevatorJavaApp( 	final String p_sdFileName,
			 						final String p_buildingFileName ) {
		sdResUri = URI.createFileURI( "./" + p_sdFileName ).resolve( BASE_URI );
		buildingModelResUri = URI.createFileURI( "./" + p_buildingFileName ).resolve( BASE_URI );
	}
	 

	public static void main( String[] args) {
		final LaunchSdmElevatorJavaApp application = new LaunchSdmElevatorJavaApp( args[ 0 ], args[ 1 ] );
		
		try {
			application.launchSdmElevator();
		}
		catch ( final Throwable p_th ) {
			p_th.printStackTrace();
		}
	}
	
	private void launchSdmElevator() 
	throws SDMException, IOException {

		// Initialize EMF for standalone execution
		final ResourceSet resourceSet = new ResourceSetImpl();
		initResourceSet( resourceSet );
		
		// Get activity of story diagram
		final Resource sdRes = resourceSet.getResource( sdResUri, true );
		
		if ( sdRes.getContents().isEmpty() ) {
			throw new UnsupportedOperationException( "Empty Story diagram!" );
		}
		final Activity activity = (Activity)  sdRes.getContents().get( 0 );
		
		// Get building object to be transformed
		final Resource buildingRes = resourceSet.getResource( buildingModelResUri, true );
		
		if ( buildingRes.getContents().isEmpty() ) {
			throw new UnsupportedOperationException( "Empty model!" );
		}
		
		final EObject building = buildingRes.getContents().get( 0 );

		// Create story diagram parameter values
		final List<Variable<EClassifier>> variableBindings = new ArrayList<Variable<EClassifier>>();
		
		final Variable<EClassifier> buildingVar = new Variable<EClassifier>( "this", building.eClass(), building );
		variableBindings.add( buildingVar );
		
		// Execute the story diagram
		new MLSDMInterpreterLauncher().launch(	activity,
												variableBindings,
												null,
												null,
												false, 
												false,
												false,
												null,
												Logger.getGlobal(),
												System.out );
		
		// Build output file uri
		final String fileName = buildingModelResUri.trimFileExtension().lastSegment();
		final URI outFileUri = buildingModelResUri.trimSegments(  1 ).appendSegment( fileName +  "_out" ).appendFileExtension( buildingModelResUri.fileExtension() );
		
		// Save output model
		final Resource outRes = resourceSet.createResource( outFileUri  );
		outRes.getContents().add( building );
		
		// Issue #2040: Necessary for when elevator ecore model is not in package registry
		outRes.save( Collections.singletonMap( XMLResource.OPTION_SCHEMA_LOCATION, Boolean.TRUE ) );
//		outRes.save( null );
	}
	
	private void initResourceSet( final ResourceSet resourceSet ) {
		final Map<String, Object> extensionToFactoryMap = resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap();
		extensionToFactoryMap.put( Resource.Factory.Registry.DEFAULT_EXTENSION, new XMIResourceFactoryImpl() );
		extensionToFactoryMap.put( sdResUri.fileExtension(), new XMIResourceFactoryImpl() );

		final EPackage.Registry packageRegistry = resourceSet.getPackageRegistry();
		packageRegistry.put( MlcorePackage.eNS_URI, MlcorePackage.eINSTANCE );
		packageRegistry.put( MlexpressionsPackage.eNS_URI, MlexpressionsPackage.eINSTANCE );
		packageRegistry.put( MlcallactionsPackage.eNS_URI, MlcallactionsPackage.eINSTANCE );
		
		packageRegistry.put( MlstorypatternsPackage.eNS_URI, MlstorypatternsPackage.eINSTANCE );
		packageRegistry.put( MlsdmPackage.eNS_URI, MlsdmPackage.eINSTANCE );
	}
}
